/**
 * Model code.
 * 
 * @author    Andy Frank
 * @creation  27 Dec 04
 */
 
////////////////////////////////////////////////////////////////
// Global
////////////////////////////////////////////////////////////////

var model = new Model();
 
////////////////////////////////////////////////////////////////
// Model
////////////////////////////////////////////////////////////////

function Model()
{
  this.add  = function(feed)  { feeds.push(feed); }
  this.get  = function(index) { return feeds[index]; }
  this.size = function() { return feeds.length; }
  var feeds = new Array();  
  
  /**
   * Return a new unique uid.
   */
  this.makeUniqueUid = function(url)
  {
    var index  = url.indexOf("://");
    var body   = url.substring(index+3);
    var domain = body.split("/")[0];
    var count = 1;
    var name = domain;
    for (var i=0; i<feeds.length; i++)
    {
      if (name == feeds[i].uid)
      {
        name = domain + (count++);
        i = -1; // reset loop
      }
    }
    return name;        
  }
  
  /**
   * Remove the given feed from the model.
   */
  this.remove = function(feed)
  {
    for (var i=0; feeds.length; i++)
      if (feed.uid == feeds[i].uid)
      {
        feeds.splice(i,1);
        return;
      }
  }
  
  /**
   * Alphatbize list by display name.
   */
  this.sort = function()
  {
    // TODO - Simple bubble sort, use
    // more effecient algoritm
    
    for (var i=0; i<feeds.length-1; i++)
      for (var j=feeds.length-1; j>i; j--)
      {
        var a = feeds[j].getDisplayName().toLowerCase();
        var b = feeds[j-1].getDisplayName().toLowerCase();
        if (a < b)
        {
          var temp = feeds[j];
          feeds[j] = feeds[j-1];
          feeds[j-1] = temp;          
        }
      }
  }
}

////////////////////////////////////////////////////////////////
// Feed Model
////////////////////////////////////////////////////////////////

function Feed()
{
  this.uid = null;
  this.url = null;
  this.defaultName = null;
  this.customeName = null;
  this.error = ERROR_OK;
  this.loaded = false;
  
  this.getDisplayName = function()
  {
    return (this.customName == null) ? this.defaultName : this.customName;
  }
  
  this.getUnread = function()
  {
    var unread = 0;
    for (var i=0; i<this.flags.length; i++)
      if ((this.flags[i] & 0x03) == 0) // unread/trash
        unread++;        
    return unread;
  }
  
  this.add = function(article) 
  { 
    articles.push(article); 
    this.flags.push(0);
  }
  
  this.addExisiting = function(article) 
  { 
    articles.push(article); 
  }
  
  this.get = function(index) 
  { 
    return articles[index]; 
  }
  
  this.remove = function(index) 
  { 
    articles.splice(index,1); 
    this.flags.splice(index,1);
  }
  
  this.size = function() { return articles.length; }
  var articles = new Array();  
  
  // Flags
  this.flags = new Array();

// TODO - make these directly off index
  this.isRead = function(article) 
  { 
    var index = getIndexOf(article);
    return ((this.flags[index] & 0x01) != 0);
  }
  this.setRead = function(article, value) 
  { 
    var index = getIndexOf(article);
    if (value) this.flags[index] |= 0x01;
    else this.flags[index] &= 0xFE;
  }

  this.isTrash = function(article) 
  { 
    var index = getIndexOf(article);
    return ((this.flags[index] & 0x02) != 0);
  }
  this.setTrash = function(article, value) 
  { 
    var index = getIndexOf(article);
    if (value) this.flags[index] |= 0x02;
    else this.flags[index] &= 0xFD;
  }
  
  this.isFlagged = function(article) 
  { 
    var index = getIndexOf(article);
    return ((this.flags[index] & 0x04) != 0);
  }
  this.setFlagged = function(article, value) 
  { 
    var index = getIndexOf(article);
    if (value) this.flags[index] |= 0x04;
    else this.flags[index] &= 0xFB;
  }
  
  function getIndexOf(article)
  {
    for (var i=0; i<articles.length; i++)
      if (article.link == articles[i].link) 
        return i;
    return null;
  }

  /**
   * Sort articles by descending date.
   */
  this.sort = function()
  {   
    // TODO - throw an exception?
    if (!this.loaded) return;
    
    // TODO - Simple bubble sort, use
    // more effecient algoritm
    
    for (var i=0; i<articles.length-1; i++)
      for (var j=articles.length-1; j>i; j--)
      {
        var a = articles[j].date;
        var b = articles[j-1].date;
             
        if (a > b)
        {
          var temp = articles[j];
          articles[j] = articles[j-1];
          articles[j-1] = temp;          
          
          var temp = this.flags[j];
          this.flags[j] = this.flags[j-1];
          this.flags[j-1] = temp;          
        }
      }
  }
}

////////////////////////////////////////////////////////////////
// Article Model
////////////////////////////////////////////////////////////////

function Article()
{
  this.link  = null;  
  this.title = null;
  this.body  = null;
  this.date  = null;
  // TODO - going bye-bye
  this.read  = true;
  this.flag  = false;  
  this.trash = false;
}
 
////////////////////////////////////////////////////////////////
// Collections
////////////////////////////////////////////////////////////////

function NormalCollection(feed)
{
  this.feed = feed;  
  var items = new Array();
  for (var i=0; i<feed.size(); i++)
    if (!feed.get(i).trash)
      items.push(feed.get(i));      
  this.get  = function(index) { return items[index]; }
  this.size = function() { return items.length; }
  this.getArray = function() { return items; }
} 

function FlaggedCollection()
{
  var items = new Array();  
  for (var i=0; i<model.size(); i++)
  {
    var feed = model.get(i);      
    for (var j=0; j<feed.size(); j++)
      if (feed.get(j).flag)
        items.push(feed.get(j));
  }    
  this.get  = function(index) { return items[index]; }
  this.size = function() { return items.length; }
  this.getArray = function() { return items; }
} 

function TrashCollection()
{
  var items = new Array();
  for (var i=0; i<model.size(); i++)
  {
    var feed = model.get(i);      
    for (var j=0; j<feed.size(); j++)
      if (feed.get(j).trash)
        items.push(feed.get(j));
  }   
  this.get  = function(index) { return items[index]; }
  this.size = function() { return items.length; }
  this.getArray = function() { return items; }
} 
 
